#-*-coding:utf-8-*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
#
###########################################################################
"""
    Outil de gestion d'onglet, profil, et item de bureau
    à partir de la description des tables "tables", "linktables"
    instancie les objets mappant les tables,
    et fournit des fonctions d'accès aux données
    contient des fonctions particulières écrites par l'utilisateur
"""
from posh.db.tables import tables, linktables
from posh.db.db import TableDBConstructor, LinkDBConstructor
from posh.config.config import profile_db_config
from traceback import print_exc
from posh.backend.error import *
globs = globals()
adddict = []

def make_get_func(linktabledatas, getlinkname):
    """
        fabrique une fonction de récupération de données liées à travers les tables de jointure
        renvoie une fonction qui:
        à l'id d'un élément de la table 'first'
        renvoie les éléments de la table 'second' qui lui sont liés
    """
    first, second = linktabledatas[0]['name'], linktabledatas[1]['name']
    firstkey, secondkey = linktabledatas[0]['idattr'], linktabledatas[1]['idattr']
    def link_func(idfirst=None):
        def _get_by_id():
            """ renvoie un dico {'iddans la première table' : [liste des éléments liés]}
            """
            links = globs[getlinkname]()
            values = {}
            for link in links:
                variable_dict = {firstkey:link[firstkey]}
                first_item = globs['get_%s' % first](**variable_dict)
                variable_dict = {secondkey:link[secondkey]}
                second_item = globs['get_%s' % second](**variable_dict)
                if first_item:
                    first_item = first_item[0]
                    if second_item:
                        second_item = second_item[0]
                    values.setdefault(first_item[firstkey], []).append(second_item)
            return values
        if idfirst:
            return _get_by_id().get(idfirst, [])
        else:
            return _get_by_id()

    return link_func

def make_set_func(linkname, get_func, add_func, del_func, tabledatas):
    """ renvoie la fonction de définition de lien
    """
    key1, key2 = tabledatas[0]['idattr'], tabledatas[1]['idattr']
    table1, table2 = tabledatas[0]['name'], tabledatas[1]['name']

    def set_func(id1, id2s):
        """ fonction de définition de lien entre id1 et les id2s
            crée les liens, supprime les liens en trop et renvoie:
            [ids des éléments ajoutés],[ids des éléments supprimés]
        """
        all_links = get_func(**{key1:id1})
        todel = [link for link in all_links if link[key2] not in id2s]
        id2s = [{key1:id1, key2:id2}for id2 in id2s]
        toadd = [link for link in id2s if link not in all_links]
        for link in todel:
            try:
                del_func(**link)
            except:
                print_exc()
                raise DelLinkDBError(linkname, table1, table2, key1, key2, link)

        for link in toadd:
            try:
                add_func(link[key1], link[key2])
            except:
                print_exc()
                raise AddLinkDBError(linkname, table1, table2, key1, key2, link)

        return [link[key2] for link in toadd], [link[key2] for link in todel]
    return set_func

def make_del_func(tablename, tabledatas, handler):
    """ supprime un élement d'une table ainsi que toutes ses références dans
        des tables de jointure
    """
    def del_func(_id):
        """
            pour chaque table de jointure, on cherche si tablename est dedans, si oui
            on fait un del sur les entrées ayant un key=id
            on del l'élément
        """

        for linkname, linktabledatas in linktables.items():
            if tablename == linktabledatas[0]['name']:
                idkey = linktabledatas[0]['idattr']
                globs['del_%s' % linkname](**{idkey:_id})
            elif tablename == linktabledatas[1]['name']:
                idkey = linktabledatas[1]['idattr']
                globs['del_%s' % linkname](**{idkey:_id})
        id_key = handler.get_id_key()

        del_item_func = getattr(handler, 'del_%s' % tablename)
        del_item_func(**{id_key:_id})
        return True
    return del_func

# gestion des tables
HANDLERS = {}
for tablename, tabledatas in tables.items():
    HANDLERS[tablename] = TableDBConstructor(profile_db_config, tabledatas)
    globs["get_%s" % tablename] = getattr(HANDLERS[tablename], "get_%s" % tablename)
    globs["add_%s" % tablename] = getattr(HANDLERS[tablename], "add_%s" % tablename)

# gestion des tables de jointure
LINKHANDLERS = {}
for linkname, linktabledatas in linktables.items():
    LINKHANDLERS[linkname] = LinkDBConstructor(profile_db_config, linktabledatas)
    # création des noms des fonctions (factorys) que l'on va proposer
    getlinkname = "get_%s_links" % linkname
    getname = "get_%s" % linkname
    addname = "add_%s" % linkname
    delname = "del_%s" % linkname
    setlink = "set_%s_links" % linkname
    # création des factorys
    globs[getlinkname] = LINKHANDLERS[linkname].get_links
    globs[getname] = make_get_func(linktabledatas, getlinkname)
    globs[addname] = LINKHANDLERS[linkname].add_link
    globs[delname] = LINKHANDLERS[linkname].del_link
    globs[setlink] = make_set_func(linkname,
                                   LINKHANDLERS[linkname].get_links,
                                   LINKHANDLERS[linkname].add_link,
                                   LINKHANDLERS[linkname].del_link,
                                   linktabledatas)

# On rajoute les fonctions de suppression d'éléments de la table
# qui intègrent les suppressions de jointure
for tablename, tabledatas in tables.items():
    globs["del_%s" % tablename] = make_del_func(tablename,
                                                tabledatas,
                                                HANDLERS[tablename])
#######################################
# FONCTIONS COMMUNES UTILISÉES PAR L'INTERFACE
#--------------------------------------
#######################################
def get_keys(_type):
    """
        renvoie les clés de la table _type avec les libellés
    """
    return HANDLERS[_type].get_keys()

def get_fields(_type):
    """
        renvoie les clés de la table _type avec les libellés
        input : _type (str)
        output : liste de dico
    """
    return HANDLERS[_type].get_fields()

def sort_by_name(a,b):
    """ fonction de tri par attribut name
    """
    return a['name'] > b['name']

def get_tables():
    """
        renvoie les descriptions des tables (renvoie une liste de dico {'libelle':'libellé de la table', 'name':'nom de la table'}
    """
    return HANDLERS.keys()
    tables = map(lambda key:{'name':key, 'libelle':HANDLERS[key].table_libelle},HANDLERS.keys())
    tables.sort(sort_by_name)
    return tables

def get_link_tables():
    """
        renvoie les descriptions des tables de jointure
    """
    return LINKHANDLERS.keys()
    tables = map(lambda key:{'name':key, 'libelle':LINKHANDLERS[key].table_libelle},LINKHANDLERS.keys())
    tables.sort(sort_by_name)
    return tables

# Actions sur les tables

def add(_type, dict_values):
    """
        ajoute un élément dans la base de données
    """
    if globs.has_key("add_%s" % _type):
        try:
            globs["add_%s" % _type](dict_values)
        except:
            print_exc()
            raise AddDBError(_type, dict_values)
    else:
        raise Exception, "Error: this add handler ('%s') is not referenced." % _type

def get(_type, dict_values={}):
    """
        renvoie les éléments de la table _type qui ont les params renseignés dans dict_values
    """
    if globs.has_key("get_%s" % _type):
        try:
            return globs["get_%s" % _type](**dict_values)
        except:
            print_exc()
            raise GetDBError(_type, dict_values)
    else:
        raise Exception, "Error: this get handler ('%s') is not referenced." % _type

def get_id_key(_type):
    """
        renvoie le nom de la clé servant d'id à une table
    """
    if _type in HANDLERS.keys():
        return HANDLERS[_type].get_id_key()
    else:
        raise Exception, "Error: this get_id_key handler ('%s')  is not referenced." % _type

def del_item(_type, _id):
    """ supprimer l'élément de la table _type d'identifiant _id
    """
    if globs.has_key("del_%s" % _type):
        try:
            return globs["del_%s" % _type](_id)
        except:
            print_exc()
            raise DelDBError(_type, _id)
    else:
        raise Exception, "Error: this del handler ('%s') is not referenced." % _type

def del_items(_type, _ids):
    for _id in _ids:
        del_item(_type, _id)
    return True

def modify(_type, _id, **dbkeys):
    """
        modifie l'entrée d'id _id avec les clés-valeurs de dbkeys
    """
    if _type in HANDLERS.keys():
        try:
            HANDLERS[_type].modify(_id, **dbkeys)
        except:
            raise ModDBError(_type, _id, dbkeys)
    else:
        raise Exception, "Error: this table ('%s') is not referenced." % _type

# Actions sur les tables de jointure
def set_links(tablename, id1, id2s):
    """
        définit les liens entre id1 et les id2s
    """
    if globs.has_key("set_%s_links" % tablename):
        return globs["set_%s_links" % tablename](id1, id2s)
    else:
        raise Exception, "Error: this set_links %s handler is not referenced." % tablename

def add_link(tablename, id1, id2):
    """
        ajoute un lien entre id1 et id2
    """
    if globs.has_key("add_%s" % tablename):
        try:
            return globs["add_%s" % tablename](id1, id2)
        except:
            print_exc()
            t_datas = get_link_datas(tablename)
            table1, table2, key1, key2 = t_datas[0]['name'], t_datas[1]['name'], t_datas[0]['idattr'], t_datas[1]['idattr']
            raise AddLinkDBError(tablename, table1, table2, key1, key2, {key1:id1,key2:id2})
    else:
        raise Exception, "Error: this add_link %s handler is not referenced." % tablename

def get_link(tablename, firstitemid=None):
    """
        renvoie les éléments liés à 'firstitemid'
        (id d'un élément du premier lien de la table de jointure)
        par la table de jointure 'tablename'
    """
    if globs.has_key("get_%s" % tablename):
        try:
            return globs["get_%s" % tablename](firstitemid)
        except:
            print_exc()
            t_datas = get_link_datas(tablename)
            table1, table2, key1, key2 = t_datas[0]['name'], t_datas[1]['name'], t_datas[0]['idattr'], t_datas[1]['idattr']
            raise GetLinkDBError(tablename, table1, table2, key1, key2, {key1:firstitemid})
    else:
        raise Exception, "Error: this get_link handler %s is not referenced." % tablename

def get_link_datas(tablename):
    """
        renvoie les descriptions de la table de jointure tablename
    """
    if linktables.has_key(tablename):
        return linktables[tablename]
    else:
        raise Exception, "Error: this linktables %s doesn't exist." % tablename


#######################################################
#   FONCTIONS PARTICULIERES
#   fonctions spécifiques à certaines tables
#######################################################
# Fonction de récupération des liens profile-user_attrs
#------------------------------------------------------
def get_all_profiles():
    """ récupération des liens profile attributs utilisateurs
        renvoie un dico {'cle ldap':{'valeur':'nom de profile'}}
    """
    all_links = LINKHANDLERS['userattrprofile'].get_links()
    all_profiles = {}
    for link in all_links:
        profileid = link['profileid']
        userattrid = link['userattrid']
        userattr = get_userattr(userattrid=userattrid)
        profile = get_profile(profileid=profileid)
        if userattr and profile:
            userattr = userattr[0]
            profile = profile[0]
            all_profiles.setdefault(userattr['attrname'], {}).setdefault(userattr['attrvalue'], []).append(profile['name'])
    return all_profiles
