#-*-coding:utf-8-*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
###########################################################################
"""
    gestionnaire de base de données pour le gestionnaire de profil du portail
    TableDBConstructor : Objet de gestion de table
    LinkDBConstructor : Objet de gestion de table de jointure
"""
from copy import deepcopy
import MySQLdb
import traceback

class DB(object):
    """
        gestionnaire de base de données
    """
    CREATE_DB = """CREATE DATABASE IF NOT EXISTS `poshprofile` DEFAULT CHARACTER SET utf8;"""
    ADD_TABLE = """CREATE TABLE IF NOT EXISTS '%s'(%s)"""

    def __init__(self, dbinfos={'host':'','user':'', 'passwd':'', 'db':''}):
        if '' in dbinfos.values():
            raise Exception, "Missing databse infos values"
        self.dbinfos = dbinfos
        self.init_db()

#    def __getattribute__(self, attrname):
#        """
#            s'assure que l'on renvoie bien une copie des attributs
#            lorsque ceux-ci ne sont pas éditables
#        """
#        print "GETATTRIBUTE : %s " % attrname
#        value = super(DB, self).__getattribute__(attrname)
#        print "Value : %s" % attrname
#        if attrname in DB.editable or type(value) == str:
#            return value
#        else:
#            return deepcopy(value)

    def init_db(self):
        """ crée la base de données si elle n'existe pas
        """
        self.connect_db(True)
        self.db.query(self.CREATE_DB)
        self.close_db()

    def connect_db(self, mysql=False):
        if mysql:
            dbinfos = self.dbinfos.copy()
            dbinfos.pop('db')
            self.db = MySQLdb.connection(**dbinfos)
        else:
            self.db = MySQLdb.connection(**self.dbinfos)

    def close_db(self):
        self.db.close()

    def add_table(self, add_table_query):
        """
            ajoute une table dans la base si elle n'existe pas encore
        """
        print "Requête Mysql (ajout de table) : %s" % add_table_query
        self.connect_db()
        self.db.query(add_table_query)
        self.close_db()

    def query(self, query_str):
        """ éxécute une requête dans la base
        """
        print "Requête Mysql (query) : %s" % query_str
        self.connect_db()
        self.db.query(query_str)
        rows = self.db.store_result()
        result = list(rows.fetch_row(0))
        self.close_db()
        return result

    def insert(self, query_str):
        """ éxécute une requête d'insert dans la base
        """
        print "Requête Mysql (insert) : %s" % query_str
        self.connect_db()
        self.db.query(query_str)
        self.close_db()
        return True

    def delete(self, query_str):
        return self.insert(query_str)

class TableDB(DB):
    """
        Modèle de gestionnaire de table sql
    """
    table_name = ""
    CREATE_TABLE = ""
    FIELDS = []
    ADDFIELD_QUERY = ""
    FIELD_QUERY = """SELECT * FROM `%s` %%s """ % table_name
    DELETE_QUERY = """DELETE FROM %s WHERE %%s """ % (table_name)

    def __init__(self, dbinfos):
        """ ajoute la table si elle existe pas encore
        """
        super(TableDB, self).__init__(dbinfos)
        self.add_table(self.CREATE_TABLE)

    def _add_field(self, field_list):
        """
            ajoute un champ dans la table
            field_list : liste des valeurs de champ
        """
        # escape les caractères
        field_list = [MySQLdb.escape_string(field) for field in field_list]
        self.insert(self.ADDFIELD_QUERY % ','.join(map(lambda x: '"%s"'%str(x), field_list) ))

    def _add_formated_field(self, field_dict):
        """
            ajoute un champ dans la table
            field_list : liste des valeurs de champ
        """
        # escape les caractères
        field_dict = dict([(key, MySQLdb.escape_string(value))for key,value in field_dict.items()])
        self.insert(self.ADDFORMATTEDFIELD_QUERY % field_dict)

    def _get_fields(self, order=None, **dbkeys):
        """ renvoie les entrée de table sql correspondant
            aux clés-valeurs indiquées dans dbkeys
            triés selon le critère order
        """
        # on filtre que les champs existent et ne sont pas vides
        for field, value in dbkeys.items():
            if field not in self.FIELDS or not value:
                dbkeys.pop(field)
        if order not in self.FIELDS and order:
            raise Exception, "Clé inexsitante : %s" % order

        # construction de la requete
        query_opts = ""
        if dbkeys:
            query_opts = 'WHERE '
            query_opts += ' and '.join(["%s='%s'"%(key, MySQLdb.escape_string(value)) for key, value in dbkeys.items()])
        if order:
            query_opts += " ORDER BY %s"%order
        query_str = self.FIELD_QUERY % query_opts
        return self.query(query_str)

    def format_field(self, field_db):
        """ formatte la sortie bdd en format utilisable par le mapper
        """
        field = dict([(self.FIELDS[indice], key)for indice, key in enumerate(field_db)])
        return field

    def _get_formated_fields(self, order=None, **dbkeys):
        """
            renvoie les champs de la base formatté en dico {clé:valeur}
        """
        return [self.format_field(field) for field in self._get_fields(order, **dbkeys)]


    def _del_field(self, **dbkeys):
        """ supprime l'entrée dans la base du thumbs thumb_name avec l'url url
        """
        # on filtre que les champs existent et ne sont pas vides
        for field, value in dbkeys.items():
            if field not in self.FIELDS or not value:
                dbkeys.pop(field)
        if dbkeys:
            query_opts = ' and '.join(["%s='%s'"%(field,
                                                  MySQLdb.escape_string(value))
                                           for field, value in dbkeys.items()])
            query_str = self.DELETE_QUERY % query_opts
            self.delete(query_str)
            return True
        raise Exception, "Error: missing parameters for deleting entry in database"

    def purge_table(self):
        """ supprime tous les éléments de la table thumbs
        """
        query_str = """DELETE FROM %s""" % self.table_name
        self.delete(query_str)
        return True

    def get_keys(self):
        """ renvoie les clés de la table sans l'id """
        return deepcopy(self.FIELDS[1:])


class TableDBConstructor(TableDB):
    """
        constructeur d'interface de base de données
        mytable: {'name':nomdelatable
                  'fields': ({'_type':type de champ,
                                           'libelle':'libellé du champ',
                                           'name':nom du champ,
                                           'default_value': valeur par défaut du champ, (facultatif
                                           'unique': 'ok' (facultatif
                                          },
                            )
                    'libelle':Intitulé de la table
                 }
    """
    ADD_FIELD_QUERY = """INSERT into `%s` (%s) VALUES (NULL, %%s)"""
    ADDFORMATTEDFIELD_QUERY = """INSERT into `%s` (%s) VALUES (NULL, %s)"""
    CREATE_TABLE = """CREATE TABLE IF NOT EXISTS `%s` (
                        %%s
    )ENGINE = MYISAM DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci COMMENT='%s posh';"""
    # FIXME:
    UPDATE_QUERY = """UPDATE `%s` SET %%s WHERE %%s='%%s'"""
    FIELD_QUERY = """SELECT * FROM `%s` %%s """
    DELETE_QUERY = """DELETE FROM %s WHERE %%s """


    def __init__(self, dbinfos, mytable):
        self.FIELDS = [field['name'] for field in mytable['fields']]
        # FIXME:
        self.modifiable = [field['name'] for field in mytable['fields']if field.has_key('modify')]

        self.fields_datas = mytable['fields']
        self.table_name = mytable['name']
        self.table_libelle = mytable['libelle']
        self.ADDFIELD_QUERY = self.ADD_FIELD_QUERY % (self.table_name,
                                                      ','.join(self.FIELDS))
        # construction de string "%(nomduchamp)s" pour les insertions dans la base
        temporary = "," . join(["'%(" + "%s" % key +")s'" for key in self.FIELDS[1:]])

        self.ADDFORMATTEDFIELD_QUERY = self.ADDFORMATTEDFIELD_QUERY % (self.table_name,
                                                                                ','.join(self.FIELDS),
                                                                                temporary)

        self.CREATE_TABLE = self.CREATE_TABLE % (self.table_name,
                                                 self.table_libelle)
        self.make_create_table_query(mytable['fields'])
        self.FIELD_QUERY = self.FIELD_QUERY % self.table_name
        self.DELETE_QUERY = self.DELETE_QUERY % self.table_name
        # FIXME :
        self.UPDATE_QUERY = self.UPDATE_QUERY % self.table_name

        # instanciation des méthodes nommées
        setattr(self, "get_%s" % self.table_name, self._get_formated_fields)
        setattr(self, "del_%s" % self.table_name, self._del_field)
        setattr(self, "add_%s" % self.table_name, self._add_formated_field)

        super(TableDBConstructor, self).__init__(dbinfos)

    def make_create_table_query(self, fields):
        """
            construit les requêtes d'initialisation de table
            pour chaque champ, gestion des types, de l'unicité
            et des valeurs par défaut
        """
        query = []
        for field in fields:
            if field['_type'] == 'autoincrement':
                q = "`%s` MEDIUMINT NOT NULL AUTO_INCREMENT PRIMARY KEY" % (field['name'])
            else:
                if 'INT' in field['_type']:
                    q = "`%(name)s` %(_type)s NOT NULL %%s" % field
                elif 'CHAR' in field['_type']:
                    q="`%(name)s` %(_type)s CHARACTER SET utf8 COLLATE utf8_bin NOT NULL %%s" % field
                if field.has_key('DEFAULT'):
                    default = "DEFAULT %s" % str(field['DEFAULT'])
                    q = q % default
                else:
                    q = q % ""
            query.append(q)
        uniques = []
        for field in fields:
            if field.has_key('UNIQUE'):
                uniques.append("`%s`" % field['name'])
        if uniques:
            unique_query = "UNIQUE(%s )" % ',\n'.join(uniques)
            query.append(unique_query)
        self.CREATE_TABLE = self.CREATE_TABLE % ',\n'.join(query)

    def get_fields(self):
        """ renvoie les champs et leur libellés
            sauf le id
        """
        return deepcopy([field for field in self.fields_datas if field['_type'] != 'autoincrement'])

    def get_id_key(self):
        """
            renvoie le nom de la clé qui sert d'identification
        """
        for field in self.fields_datas:
            if field['_type'] == 'autoincrement':
                return field['name']
        else:
            return "id"

    def modify(self, _id, **dbkeys):
        """
            modifie une valeur pour l'entrée de la table d'id _id
        """
        idkey = self.get_id_key()
        args = ''
        for key, value in dbkeys.items():
            if key not in self.modifiable:
                raise Exception, "Erreur de mofication d'entrée :  Cette clé n'est pas référencée"
            if args: args+=","
            args += "%s='%s'" % (key, MySQLdb.escape_string(value))
        update_query = self.UPDATE_QUERY %(args, idkey, _id)
        self.insert(update_query)

#################################################
# GESTIONNAIRE POUR LES TABLES DE JOINTURE
#################################################
class LinkDB(TableDB):
    """
        Modèle de table de link
    """
    table_name = ''
    FIELDS = ["id1", "id2"]
    ADDFIELD_QUERY = """INSERT into `%s` (%s) VALUES (%%s)""" % (table_name,
                                                                 ','.join(FIELDS))
    CREATE_TABLE = """CREATE TABLE IF NOT EXISTS `%s` (
                        `id1` MEDIUMINT NOT NULL COMMENT 'identificateur 1',
                        `id2` MEDIUMINT NOT NULL COMMENT 'identificateur 2'
                      )ENGINE = MYISAM DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci COMMENT='table modele de lien';""" % table_name
    FIELD_QUERY = """SELECT * FROM `%s` %%s """ % table_name
    DELETE_QUERY = """DELETE FROM %s WHERE %%s """ % (table_name)

    def add_link(self, id1, id2=None):
        """ rajoute un lien entre l'élément id1 et l'élément id2
        """
        link = [id1, id2]
        self._add_field(link)
        return True

    def get_links(self, **attrs):
        """
            liste les liens liés à id1 ou id2 ou les 2
            et les renvoie sous une forme [{'id1':valeur, 'id2':valeur}]
        """
        if not set(attrs.keys()).issubset(set(self.FIELDS)):
            raise Exception, "Erreur : requête vers des clés non existante."
        return [self.format_field(field_db) for field_db in self._get_fields(**attrs)]

    def del_link(self, **attrs):
        """
            supprime un lien entre id1 et id2
        """
        if not set(attrs.keys()).issubset(set(self.FIELDS)):
            raise Exception, "Erreur : requête vers des clés non existante."
        return self._del_field(**attrs)


class LinkDBConstructor(LinkDB):
    """
        classe de gestion de table de jointure
        mytable = [{'name':nomdelapremièretable, 'idattr': nom du champ},
                   {'name':nomdeladeuxièmetable, 'idattr':'nom du champ id dans la deuxième table}
                   ]
        la table de jointure s'appelera :
        nomdelapremièretablenomdeladeuxièmetable
    """
    CREATE_TABLE = """CREATE TABLE IF NOT EXISTS `%s` (
                        `%s` MEDIUMINT NOT NULL,
                        `%s` MEDIUMINT NOT NULL
                      )ENGINE = MYISAM DEFAULT CHARSET=utf8 COLLATE=utf8_unicode_ci COMMENT='';"""
    ADDFIELD_QUERY = """INSERT into `%s` (%s) VALUES (%%s)"""
    FIELD_QUERY = """SELECT * FROM `%s` %%s """
    DELETE_QUERY = """DELETE FROM %s WHERE %%s """

    def __init__(self, dbinfos, mytable):
        self.table_name = "%s%s" % (mytable[0]['name'], mytable[1]['name'])
        self.FIELDS = [mytable[0]['idattr'], mytable[1]['idattr']]
        self.ADDFIELD_QUERY = self.ADDFIELD_QUERY % (self.table_name,
                                                ','.join(self.FIELDS))
        self.CREATE_TABLE = self.CREATE_TABLE % (self.table_name, self.FIELDS[0], self.FIELDS[1])
        self.FIELD_QUERY = self.FIELD_QUERY % self.table_name

        self.DELETE_QUERY = self.DELETE_QUERY % self.table_name
        self.table_libelle = "%s <=> %s" % (self.FIELDS[0], self.FIELDS[1])
        super(LinkDBConstructor, self).__init__(dbinfos)
