#-*-coding:utf-8-*-
"""
    description des tables de la base de données poshprofile
    les descriptions concernent aussi bien la création de la table
    que la mise en forme dans l'interface

    Schéma d'une table classique:
    ----------------------------
       {'name':"nom de la table",
        'libelle':"Intitulé de la table",
        'fields':( des champs de la table )
       }

       les champs doivent obligatoirement contenir un champ 'id' et un champ 'name' unique.

        un champ:
        ---------
       {'name':'nom du champ',
        '_type':autoincrement ou intitulé Mysql du type,
        'libelle':"Libellé du champ",
        'DEFAULT':'valeur par défaut du champ' (rien si pas de valeur),                                  (Facultatif
        'UNIQUE':'ok' (Unicité de la variable),                                                          (Facultatif
        'type_valid':liste de type à valider parmi : required/alpha/alphanum/integer/real/date/email/url (Facultatif
                    voir interface/html_intrface/scripts/fValidator-full.js pour plus de détails
       }

    Schéma d'une table de jointure:
    -------------------------------
      [{'name':nomdelapremièretable, 'idattr': nom du champ},
       {'name':nomdeladeuxièmetable, 'idattr':'nom du champ id dans la deuxième table}
      ]

"""
profiletable = {'name':"profile",
                'libelle':'Profils',
                'fields': ({'name':'profileid', '_type':'autoincrement', 'libelle':'id du profil'},
                       {'name':'name', '_type':'VARCHAR( 32 )', 'libelle':'Nom du profil', 'UNIQUE':'TRUE',
                           'type_valid':["required"]},
                       {'name':'description', '_type':'VARCHAR( 32 )', 'libelle' : 'description du profil',
                           'type_valid':["required"], 'modify':'true'}
                      ),
                }

thumbtable = {'name':"thumb",
              "libelle":"Onglets",
              'fields': ({'name':'thumbid', '_type':'autoincrement', 'libelle':"id de l'onglet"},
                         {'name':'name', '_type':'VARCHAR( 32 )', 'libelle':"Nom de l'onglet", 'UNIQUE':'TRUE',
                             'type_valid':["required"]},
                         {'name':'description', '_type':'VARCHAR( 32 )', 'libelle':"Libellé de l'onglet",
                             'type_valid':["required"], 'modify':'true'},
                         {'name':'indice', '_type':'MEDIUMINT', 'libelle':"placement de l'onglet",
                             'type_valid':["required"], 'modify':'true'},
                         {'name':'poshtype', '_type':'SMALLINT', 'libelle':"type posh de l'onglet", "DEFAULT":"2",
                             'type_valid':["required", "integer"]},
                         {'name':'url', '_type':'VARCHAR( 150 )', 'libelle':"url de l'application associée à l'onglet",
                             'type_valid':['url'], 'modify':'true'},
                         {'name':'admin_profile_id', '_type':'TINYINT(3)', 'libelle':"id de l'onglet dans le portail de  l'admin(pour page interne)",
                             "DEFAULT":"-1"
                             }
                         ),
              }
desktoptable = {'name':"desktopitem",
                "libelle":"Items de bureau",
                'fields': ({'name':'desktopid', '_type':'autoincrement', 'libelle':"id l'item de bureau"},
                           {'name':'name', '_type':'VARCHAR( 32 )', 'libelle':"nom du bouton", 'UNIQUE':'TRUE',
                               'type_valid':["required"]},
                           {'name':'libelle', '_type':'VARCHAR( 32 )', 'libelle':"libellé du bouton",
                               'type_valid':["required"], 'modify':'true'},
                           {'name':'url', '_type':'VARCHAR( 150 )', 'libelle':"url de l'application associée au bouton",
                               'type_valid':["required", 'url'], 'modify':'true'},
                           {'name':'icon', '_type':'VARCHAR( 150 )', 'libelle':"url de l'icone du bouton",
                               'modify':'true'},
                           ),
                }
userattrtable = {'name':"userattr",
                 "libelle":"Attributs utilisateur",
                 'fields': ( {'name':'userattrid', '_type':'autoincrement', 'libelle':"id de l'attribut utilisateur"},
                            {'name':'attrname', '_type':'VARCHAR( 32 )', 'libelle':"nom de l'attribut utilisateur",
                                'type_valid':["required", 'alphanum'], 'modify':'true'
                                },
                            {'name':'attrvalue', '_type':'VARCHAR( 32 )', 'libelle':"valeur de l'attribut",
                                'type_valid':["required", 'alphanum'], 'modify':'true'},
                          ),
            }
tables = {'profile':profiletable,
          'thumb':thumbtable,
          'desktopitem':desktoptable,
          'userattr':userattrtable,
            }
linktables = {'profilethumb':                   [{'name':"profile", "idattr":"profileid"},
                                                 {'name':"thumb", "idattr":"thumbid"}],
              'profiledesktopitem':             [{'name':"profile", "idattr":"profileid"},
                                                 {'name':"desktopitem", "idattr":"desktopid"}],
              'userattrprofile':                [{'name':"userattr", "idattr":"userattrid"},
                                                 {'name':"profile", "idattr":"profileid"}],
              }

