# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# parse les fichiers .ini
#
###########################################################################
"""parse les fichiers .ini
(pour l'instant perm.ini)
"""
from ConfigParser import ConfigParser
from sets import Set
from os.path import isfile


class PermissionParser(ConfigParser):
    """parse un fichier .ini de permissions"""
    def parse_file(self, filename):
        """charge les permissions depuis un fichier"""
        self.read(filename)

    def get_profiles(self):
        """affecte un """
        return dict([(name, descr) for name, descr
                                in self.items('profiles')])
    def get_permissions(self):
        """renvoie la liste des permissions définies dans le .ini
        """
        self._permissions = {}
        for thumb_name, profilenames in self.items('permissions'):
            for profile in profilenames.split(','):
                profile = profile.strip()
                self._permissions.setdefault(thumb_name, []).append(profile)

        self._permissions = self._transf_perms(self._permissions)
        return self._permissions

    def get_baseprofiles(self):
        """renvoie la liste des profiles de base définies dans le .ini
        """
        self._baseprofiles = {}
        for profile, baseprofiles in self.items('baseprofiles'):
            for baseprofile in baseprofiles.split(','):
                baseprofile = baseprofile.strip()
                self._baseprofiles.setdefault(profile, []).append(baseprofile)
        return self._baseprofiles

    def _transf_perms(self, trs_perms):
        """inverse le dictionnaire self.permissions
        d'un dico : onglet -> [profiles]
        on aboutit à un dico :
        profil -> [onglets]
        """
        trs_perms = {}
        for thumb, profiles in self._permissions.items():
            for profile in profiles:
                if trs_perms.has_key(profile):
                    trs_perms[profile].append(thumb)
                else:
                    trs_perms[profile] = [thumb]
        return trs_perms

    def get_thumbs(self):
        """renvoie la liste des onglets"""
        thumbs = {}
        for thumbname, data in self.items('thumbs'):
            description, seq, _type, param = data.split(',')
            thumbs[thumbname] = dict( name=thumbname,
                                      description=description,
                                      seq=seq,
                                      _type=_type,
                                      param=param)
        return thumbs

    def get_desktop_items(self):
        """
            renvoie la liste des boutons de bureau
        """
        desktopitems = {}
        for itemname, data in self.items('desktop'):
            libelle, url, icon = data.split(',')
            desktopitems[itemname] = dict(libelle=libelle,
                                          url=url,
                                          icon=icon)
        return desktopitems

#    def get_users(self):
#        """ retourne la liste des utilisateurs """
#        users = []
#        for username, role in self.items('pam'):
#            users.append(username)
#        return users

class ProfilerParser(ConfigParser):
    """parse un fichier .ini d'affectation de profil
    """

    def parse_file(self, filename):
        """charge les permissions depuis un fichier"""
        self.read(filename)

    def get_values(self):
        values = {}
        for section in self.sections():
            values[section] =  dict([(key, value.split(',')) for key, value
                                        in self.items(section)])
        return values
