# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# definition des acteurs
#
###########################################################################

"""Ce module contient la définition des acteurs (au sens user qui dispose
de differents onglets)
"""
from sets import Set

#class BaseProfile:
#    # FIXME : profil de base (meta_role)
#    def __init__(self, name, description="", rules=()):
#        """
#        name: Le nom du profil
#        description: description du rôle
#        rules: Un iterable qui parcours la liste des rôles à partir desquels
#                   est construit le rôle courant
#        """
#        self.name = name
#        self.description = description
#        self.rules = rules or Set()

# FIXME ajouter automatiquement des regles (un addrule dans le set)
# depuis les règles du profil de base

class Profile:
    """Définition de ce qu'est un profil"""
    def __init__(self, name, description="", rules=(), baseprofiles=()):
        """
        name: Le nom du profil
        description: description du rôle
        rules: Un iterable qui parcours la liste des rôles à partir desquels
                   est construit le rôle courant
        """
        self.name = name
        self.description = description
        self.rules = rules or Set()
        self.baseprofiles = baseprofiles or Set()

    def __str__(self):
        return self.name

    def __repr__(self):
        return '<Profile %s at %s>' % (self.name, hex(id(self)))

    def add_baseprofile(self, profile):
        """ajoute un profil de base à l'ensemble des profils de base existants
        """
        self.baseprofiles.add(profile)

    def has_baseprofile(self):
        "renvoie True ou False suivant si <self> est basé sur d'autres profiles"
        return len(self.baseprofiles) > 0

    def profile_based_on(self, profile):
        """renvoie True ou False suivant si <self> est basé sur <profile>"""
        return profile in self.baseprofiles

    def get_rules(self):
        """ collecte recursivement toutes les regles
        pas seulement les regles de <self> mais aussi les regles
        des profils de base
        """
        totalrules = self.rules
        for baseprofile in self.baseprofiles:
            totalrules = totalrules.union(baseprofile.get_rules())
        return totalrules

    def add_rule(self, rule):
        "rule: un objet Rule()"
        self.rules.add(rule)

    def add_rules(self, rules):
        "rules: Set(Rule())"
        for rule in rules:
            self.add_rule(rule)

    def remove_rule(self, rule):
        self.rules.remove(rule)

class User:
    """Utilisateur

    Un utilisateur peut avoir un ou plusieurs rôles.
    Les autorisations peuvent être attribuées soit à un rôle,
    soit à utilisateur.

    Une autorisation donnée à un utilisateur a plus de priorité
    qu'une autorisation donnée à l'un de ses rôles
    """
    def __init__(self, name, user_datas={}, profiles=None):
        self.name = name
        self.user_datas = user_datas
        self.profiles = profiles or Set()

    def add_profile(self, profile):
        """Donne un profil supplémentaire à l'utilisateur
        """
        self.profiles.add(profile)

    def remove_profile(self, profile):
        """Retire à un profil à l'utilisateur"""
        try:
            self.profiles.remove(profile)
        except KeyError:
            print "Profil '%s' pas attribué a %s" % (profile, self)

    def __str__(self):
        return self.name

