#!/usr/bin/python
#-*-coding:utf-8-*-
"""
    client xmlrpc renvoyant les description des onglets de l'utilisateur posh courant
    appel avec :
        python client.py <infos utilisateur au format Json>

"""
import commands
import json
import xmlrpclib
import sys
from posh.config.config import XMLRPC_IP_SERVER, XMLRPC_PORT, LOCAL_IP
from posh.thumbs.initthumbs import InitThumbs
from posh.thumbs.exportthumbs import ExportThumbs

debug = True

def debug(msg):
    if debug:
        f_ = file('/tmp/client_desktop.debug.log', 'a')
        f_.write(msg)
        f_.write('*******\n')
        f_.close()

def encode_value(result):
    try:
        return result.encode('utf-8')
    except:
        return result

def return_result(result):
    """
        renvoie le résultat sous forme de print pour passage au php
    """
    result = encode_value(result).strip()
    debug("++++ On renvoie un résultat au php : ")
    debug(result)
    print result

class EnvoleProfilClient:
    """
        client pour les appels profil utilisateur
        desktop: bureau utilisateur
        thumbs : les onglets de l'utilisateur
    """

    def __init__(self, arguments):
        try:
            self.opt_parse(arguments)
            self.make_server()
        except Exception, msg:
            debug(msg.message)

    def user_datas_opt_parse(self, arguments):
        """
            parse des options contenant des attributs utilisateurs
        """
        self.user_datas = self.decode_user_datas(arguments[2])
        self.username = self.user_datas['user']
        if self.user_datas.has_key('details'):
            self.user_datas = self.user_datas['details']
        if self.user_datas.has_key('infos'):
            self.user_datas = self.user_datas['infos']
        debug("   - username = %s \n\n   - user_datas = %s" % (self.username, self.user_datas))

    def thumb_exporter_opt_parse(self, arguments):
        """
            parser pour l'export d'onglet
        """
        self.username, self.user_id, self.thumb_id, self.thumb_name, self.thumb_indice = arguments[2:]
        debug("    - user_id  = %s \n\n   - thumb_id = %s \n,  - thumb_name : %s\n  \
                - thumb_indice: %s" % (
                        self.user_id, self.thumb_id, self.thumb_name, self.thumb_indice))


    def opt_parse(self, arguments):
        """
            parse les arguments passés à l'éxécution du fichier
            récupère le mode et parse selon le mode
        """
        debug(str(arguments))
        parser = {'desktop':self.user_datas_opt_parse,
                  'thumbs':self.user_datas_opt_parse,
                  'exporter':self.thumb_exporter_opt_parse,}
        self.mode = arguments[1]
        debug("**** On appelle l'action %s avec les paramètres: \n\n" % self.mode)
        parser.get(self.mode, self.user_datas_opt_parse)(arguments)

    def make_server(self, ip=XMLRPC_IP_SERVER, port=XMLRPC_PORT):
        """
            instancie un client xmlrpc pour appeler le UserContext
        """
        self.server = xmlrpclib.Server('http://%s:%s/' % (ip,port))
        self.key = self.server.register_frontend(LOCAL_IP)

    def decode_user_datas(self, user_datas_str):
        """
            décode les données utilisateurs envoyées au format Json
        """
        return json.read(user_datas_str)

    def get_desktop_items(self):
        """
            appel les infos pour la mise en forme des boutons du bureau utilisateur
            et les renvoie au flux de retour (print)
        """
        result = self.server.get_desktop_items(self.username, self.user_datas)
        return_result(result)

    def init_thumbs(self):
        """
            appel l'initialisation des onglets utilisateurs dans la bdd
        """
        initializer = InitThumbs(self.user_datas, self.username, self.server)
        initializer.insert_tabs_and_modules()

    def export_thumb(self):
        """
            exporte un onglet
        """
        exporter = ExportThumbs(self.username, self.user_id, self.thumb_id, self.thumb_name,
                                self.thumb_indice, self.server, self.get_my_ip() , self.get_my_key())
        ret = exporter.export_thumb()
        return_result(ret)
        #return_result(exporter._get_thumb())

    def execute_action(self):
        """
            éxécute l'action commandée
        """
        request_keys = {'desktop':self.get_desktop_items,
                        'thumbs':self.init_thumbs,
                        'exporter':self.export_thumb}
        if self.mode in request_keys.keys():
            request_keys[self.mode]()
        else:
            raise Exception, "Unknown procedure Called : %s "% self.mode

    def get_my_key(self):
        return self.key

    def get_my_ip(self):
        return LOCAL_IP

if __name__ == '__main__':
    try:
        client = EnvoleProfilClient(sys.argv)
        client.execute_action()
    except Exception, msg:
        debug(msg.message)
