#-*-coding:utf-8-*-
"""
    Fournisseur de données pour la mise en forme
"""
from posh.backend import db_mapper
from html_lib import form_title, choice_select, transfer_select, menu
from html_lib import get_key_libelle, get_available_libelle, get_linked_libelle

def add_menu():
    """
        renvoie les données pour la mise en forme du menu d'ajout de données
    """
    return menu(db_mapper.get_tables())

def link_menu():
    """
        renvoie les données pour la mise en forme du menu de lien de table
    """
    return menu(db_mapper.get_link_tables())

def add_form(_type='profiles'):
    """
        renvoie les données pour la mise en forme d'un formulaire d'ajout
        d'élément du type : _type parmi ['thumb', 'desktopitem', 'profile', 'userattr']
    """
    title = form_title(_type)
    form_name = "form_%s" % _type
    form = {}
    inputs = db_mapper.get_fields(_type)
    # on formatte les clés type_valid pour l'écriture au format du validateur javascript
    # la liste ['type1', 'type2'] --->  la string "'type1','type2'"
    for _inp in inputs:
        if _inp.has_key('type_valid'):
            if type(_inp['type_valid']) == list:
                _inp['type_valid'] = ",".join(map(lambda x:"'%s'" % str(x),_inp['type_valid']))
            else:
                _inp['type_valid'] = "'%s'" % _inp['type_valid']
        else:
            _inp['type_valid'] =""
        if _inp.has_key('DEFAULT'):
            _inp['value'] = _inp.pop('DEFAULT')
        else:
            _inp['value'] = ''
        _inp['libelle'] = get_key_libelle(_inp['name'])
    form['balises'] = inputs
    form['name'] = form_name
    form['title'] = title
    return form

def link_form(_type, current=None):
    """
        renvoie les données pour la mise en forme de formulaire de liaison
        _type ou type est le nom de la table de jointure
        current est l'id de l'élément de la première table auquel
        on veut associer des éléments de la deuxième, il vaut -1 pour rien

        on renvoie la description pour trois formulaires:
        - un select pour le choix du 'current' auxquel on veut associer des éléments (main)
        - un select pour les éléments déjà associé à 'current'                       (linked)
        - un select pour les éléments disponibles pour association                   (available)
    """
    title = form_title(_type)
    linktabledatas = db_mapper.get_link_datas(_type)
    # recupération des noms des deux tables jointes
    table1 = linktabledatas[0]['name']
    table2 = linktabledatas[1]['name']
    id1 = linktabledatas[0]['idattr']
    id2 = linktabledatas[1]['idattr']
    # récupération des valeurs de la table 1 et formatage
    main = db_mapper.get(table1)
    main_options = choice_select(table1, main, current)

    # récupération des éléments de la table 2 liés à l'élément de la table 1 en cours d'édition
    if current:
        linked = db_mapper.get_link(_type, current)
        linked_options = transfer_select(table2, linked)

        # récupération des éléments de la table 2 qui ne sont pas encore liés à l'élément courant
        all = db_mapper.get(table2)
        for elem in linked:
            if elem in all:
                all.remove(elem)
        available = all
        available_options = transfer_select(table2, available)
    else:
        available_options = []
        linked_options = []

    form_name = "form_%s" % _type
    form = {}
    return {'choice_select':{'options':main_options},
            'linked_select':{'options':linked_options},
            'available_select':{'options':available_options},
            'title':title,
            'name':form_name,
            'id1':id1,
            'id2':id2,
            "libelle1":get_linked_libelle(table1,table2),
            "libelle2":get_available_libelle(table2),
            }

def list_form(_type, sort_key='id'):
    """
        renvoie les données pour la mise en forme des entrées
        de la table de _type pour permettre la suppression (et la modification)
    """
    def sort_by_key(key):
        def sort_by(a,b):
            val1, val2 = a.get(key, 0), b.get(key,0)
            if val1 > val2:
                return 1
            else:
                return -1
        return sort_by

    title = form_title(_type)
    form_name = "form_%s" % _type
    datas = {}

    fields = db_mapper.get_fields(_type) # description des champs de la table
    for field in fields:
        field['libelle'] = get_key_libelle(field['name'])
    db_values = db_mapper.get(_type)
    id_key = db_mapper.get_id_key(_type)

    ihm_values = []

    for db_value in db_values:
        ihm_value = {'id':db_value.pop(id_key)}
        ihm_value.update(db_value)
        ihm_values.append(ihm_value)
    ihm_values.sort(sort_by_key(sort_key))

    return dict(values=ihm_values,
                keys=[field['name'] for field in fields],
                header=fields,
                )

def mod_form(_type, dbkey, item_id):
    """
        renvoie les données pour la mise en forme dun formulaire de modification
        d'éntrée de la table _type
    """
    form_name = 'form_modify'
    myfield = {}
    fields = db_mapper.get_fields(_type) # description des champs de la table
    for field in fields:
        field['libelle'] = get_key_libelle(field['name'])
        if field['name'] == dbkey:
            myfield = field
#        if not field.has_key('type_valid'):
#            field['type_valid'] = 'required'
        if field.has_key('type_valid'):
            if type(field['type_valid']) == list:
                field['type_valid'] = ",".join(map(lambda x:"'%s'" % str(x),field['type_valid']))
            else:
                field['type_valid'] = "'%s'" % str(field['type_valid'])
        else:
            field['type_valid'] =""

    value = db_mapper.get(_type, {db_mapper.get_id_key(_type):item_id})[0][dbkey]
    myfield['value'] = value

    return dict(_input=myfield,
                form_name=form_name)

