#-*- coding: iso-8859-15 -*-
"""
menu : gnrateur de menu
"""
from nevow import loaders, athena
from nevow.flat import flatten
from nevow import tags as T, rend

class BaseMenu(rend.Fragment):
    """ menu de base """
    docFactory = loaders.xmlstr('''
                            <span xmlns:nevow="http://nevow.com/ns/nevow/0.1">
                            <span nevow:render="menu"/>
                           </span>
                          ''')
    def render_menu(self, ctx, data):
        """ rend un menu"""
        link = T.div(render=T.directive('header'))
        items = T.div(render=T.directive('items'))
        return [link, items]

    def render_header(self, ctx, data):
        """rend le header """
        raise NotImplementedError("BaseMenu must be implemented to be rendered")
    def render_items(self, ctx, data):
        """ rend l'ensemble des items """
        raise NotImplementedError("BaseMenu must be implemented to be rendered")


class ListMenu(BaseMenu):
    """ menu sous forme de liste"""
    def __init__(self):
        super(ListMenu,self).__init__()
        self.items = []

    def add_item(self, data, link="#", description=""):
        """ ajoute un item au menu en cour de construction """
        self.items.append(ListItem(data, link, description, _class="list_menu_item"))

    def set_layout(self, layout):
        """ dfinit un layout pour le rendu """
        self.layout = layout

    def render_menu(self, ctx, data):
        """ rend un menu"""
        return T.div(_class="list_menu")[self.layout.place(self.items)]

class ScrollMenu(BaseMenu):
    """ menu scrollable """
    def __init__(self, title, description='', number=0):
        super(ScrollMenu,self).__init__()
        self.title = title
        self.description = description
        self.items = []
        self.number = number

    def add_item(self, data, link="#", description=""):
        self.items.append(ScrollItem(data, link, description))

    def render_header(self, ctx, data):
        """ rend l'entete du menu """
        tag = T.p(_id="scrollMenu%d"%self.number, _class="scrollMenu", onmouseover="MontrerMenu('scrollSsMenu%d');"%self.number, onmouseout="CacherDelai();", title=self.description)[
                                        T.a(href="#", onmouseover="MontrerMenu('scrollSsMenu%d');"%self.number,onfocus="MontrerMenu('scrollSsMenu%d');"%self.number)[self.title]
                             ]
        return tag

    def render_items(self, ctx, data):
        tag = T.ul(_id="scrollSsMenu%d"%self.number, _class="scrollSsMenu", onmouseover="AnnulerCacher();", onmouseout="CacherDelai();", onfocus="AnnulerCacher();", onblur="CacherDelai();")[self.items]
        return tag


##                          ITEMS


class BaseItem(rend.Fragment):
    """ item de base d'un menu
        :link: lien html correspondant
    """
    docFactory = loaders.xmlstr('''
                    <span xmlns:nevow="http://nevow.com/ns/nevow/0.1" nevow:render="baseitem">
                    </span>
            ''')
    def __init__(self, data, link="#", description="", _class=None):
        self.link = link
        self.data = data
        self.description = description
        self._class = _class

    def render_baseitem(self, ctx, data):
        """ rend un item de base pour un menu """
        tag = T.span(render=T.directive("link"))
        if self._class is not None:
            tag.attributes["_class"] = self._class
        return tag

    def render_link(self, ctx, data):
        """ rend le lien d'un item """
        return BaseLink(self.data, self.link, self.description)

class ListItem(BaseItem):
    """ item d'un menu sous forme de liste """
    def render_link(self, ctx, data):
        """ renvoit un lien """
        return T.div[BaseLink(self.data, self.link, self.description)]

class AccesItem(BaseItem):
    """ item pour un menu d'accs """
    def render_link(self, ctx, data):
        """ renvoit un lien"""
        return T.p(_class="acces_menu_items")[BaseLink(self.data, self.link, self.description)]

class ScrollItem(BaseItem):
    """ item pour un menu scrollable """
    def render_link(self, ctx ,data):
        return T.li[BaseLink(self.data, self.link, self.description)]

class ImageItem(BaseItem):
    """ item avec image insre en arrire plan et mouseover"""

    def __init__(self, data, link, description='', name=None):
        super(ImageItem, self).__init__(data, link, description)
        self.img = None
        self.over_img = None
        self.name = name

    def set_bkg(self, img_url):
        """ ajoute un arrire-plan  notre lien
           :img_url: url de l'image
        """
        self.img = img_url

    def set_description(self, description):
        """ ajoute une description au lien """
        self.description = description

    def set_over_bkg(self, img_url):
        """ ajoute un arrire-plan  notre lien pour le survol
            :img_url: url de l'image
        """
        self.over_img = img_url

    def render_link(self, ctx, data):
        """ renvoit le lien de l'item """
        return ImgLink(self.data, link=self.link, img=self.img, over_img=self.over_img, name=self.name, description=self.description)

##                        LINKS


class BaseLink(rend.Fragment):
    """ lien de base d'un menu """
    docFactory = loaders.xmlstr('''
                    <a xmlns:nevow="http://nevow.com/ns/nevow/0.1" nevow:render="link">
                    </a>
                    ''')

    def __init__(self, data, link="#", description=''):
        super(BaseLink, self).__init__()
        self.link = link
        self.data = data
        self.description = description

    def render_link(self, ctx, data):
        """ rend le lien"""
        tag = T.a()
        tag.attributes = self.render_attributes()
        tag.children = self.render_data()
        return tag


    def render_attributes(self):
        """ rend les attributs de la balise de lien """
        dico = {"href":self.link, "title":self.description}
        return dico

    def render_data(self):
        """ rend le libelle du lien """
        return self.data

class ImgLink(BaseLink):
    """ renvoit un lien avec une image en arrire plan qui change au survol
        :data:libelle du lien
        :link:lien
        :img: url de l'image d'arrire plan
        :img_over: url de l'image pour le mouseover
        :name: nom de la balise !! nom unique au sein de la page( = data par dfaut)
    """

    def __init__(self, data, link="#", description='', img="#", over_img="#", name=None):
        super(ImgLink, self).__init__(data, link, description)
        self.name = name
        self.img = img
        self.over_img = over_img

    def render_data(self):
        """ rend le contenu de la balise de lien """
        if self.img is not None:
            image = T.img(src=self.img, name=self.name)
        else:
            raise NotImplementedError("a picture must be added to the standard link")
        texte = T.p[self.data]
        return [image, texte]

    def render_attributes(self):
        """ rend les attributs de notre balise """
        dico = {'href':self.link, "title":self.description, 'name':self.name}
        if self.over_img is not None:
            dico["onMouseOut"] = "MM_swapImgRestore()"
            dico["onMouseOver"] = "MM_swapImage('%s','','%s',1)"% (self.name, self.over_img)
        return dico

#a = ScrollMenu("mon menu")
#a.add_item('libelle', "#", "cliquez-ici")
#a.add_item('google', "http://www.google.fr", "moteur de recherche")
#b = ImageItem(data="lien", link="http://point2vues.free.fr/water/", name="monlien")
#b.set_description("la description de mon lien")
#b.set_bkg("http://point2vues.free.fr/water/water.jpg")
#b.set_over_bkg("http://point2vues.free.fr/bastien/KniKul_over.jpg")
#a = BaseLink("lien")
#print flatten(a)
