# -*- coding: utf-8 -*-
from os.path import join

from nevow.rend import Page
from nevow import loaders, tags as T, inevow


from interface.interface.frags.message import Message
from interface.interface.data_handler import datas, valid
from interface.interface.tools import _get_root_url
from interface.interface.config import HTML_DIR

class ValidLinkForm(Message):
    """
        fragment de validation d'ajout d'entrée dans les tables de jointure
        de la base de données
    """
    close_btn_code = "setContent('linkform_msg_container','');"

    def __init__(self, _type, form_datas, current ):
        self._type = _type
        self.form_datas = form_datas
        self.current = current
        super(ValidLinkForm, self).__init__()

    def locateChild(self, ctx, segments):
        return self, ()

    def data_message(self, ctx, data):
        """
            valide le formulaire d'ajout
        """
        if not self.current:
            raise Exception, "Il manque des informations pour la liaison %s" % self._type
        request = inevow.IRequest(ctx)
        if request.args.has_key('linked_select'):
            linked_elements_ids = request.args['linked_select']
        else:
            linked_elements_ids = []
        choice_options = self.form_datas['choice_select']['options']
        all_tolink_elements = self.form_datas['available_select']['options']
        all_tolink_elements.extend(self.form_datas['linked_select']['options'])
        datas = valid.set_links(self._type, self.current, linked_elements_ids,
                                choice_options,
                                all_tolink_elements)
        return datas

class ChoiceSelect(Page):
    """
        Select pour les éléments à éditer
    """
    docFactory = loaders.xmlfile(join(HTML_DIR, 'choice_select.xml'))
    myurl = ['link', 'choice', '?linkform=%s']
    onchange = "var choice_select_selected_value = this.options[this.selectedIndex].value;\
                Refresh('available_widget', [['current', choice_select_selected_value]]);\
                Refresh('linked_widget', [['current', choice_select_selected_value]]);"

    myselect_id = 'choice_select'

    def __init__(self, _type, form_datas, current=None):
        self.current = current
        self._type = _type
        self.form_datas = form_datas
        Page.__init__(self)

    def locateChild(self, ctx, segments):
        return self, ()

    def make_myurl(self, ctx):
        """
            renvoie l'url du widget (celle qui le renvoie)
        """
        myurl = join(_get_root_url(ctx), *self.myurl) % self._type
        if self.current:
            return "%s&current=%s" % (myurl , self.current)
        return myurl

    def render_url_slot(self, ctx, data):
        """
            rend la valeur de l'url correspondant au widget
        """
        ctx.fillSlots('url', self.make_myurl(ctx))
        return ctx.tag

    def render_select(self, ctx, data):
        return T.select(onchange=self.onchange,_id=self.myselect_id)[T.span(data=T.directive('options'),
                                                                     render=T.directive('options'))]#self.options()]

    def data_options(self, ctx, data):
        return self.form_datas['choice_select']['options']

    def render_options(self, ctx, options):
        """
            renvoie les données pour la mise en forme des options
        """
        for option in options:
            option_tag = T.option(value=option['value'])[option['libelle']]
            if option.has_key('selected'):
                option_tag.attributes['selected'] = 'true'
            yield option_tag

class AvailableSelect(ChoiceSelect):
    """
        Select pour les éléments disponibles
    """
    docFactory = loaders.xmlfile(join(HTML_DIR, 'available_select.xml'))
    myurl = ['link', 'available', '?linkform=%s']
    myselect_id = 'available_select'

    def data_options(self, ctx, data):
        return self.form_datas['available_select']['options']

# debut de mise en forme tableau cf le javascript dans 1_ajax.js
#    def render_options(self, ctx, options):
#        for option in options:
#            yield T.tr()[
#                         T.td()[T.input(_type="hidden", value=option['value'])],
#                         T.td()[option['libelle']],
#                         T.td()[T.a(href="javascript:window.alert('delete');")[T.div[T.img(src='/image/close.gif')]]],
#                        ]
#
    def render_select(self, ctx, data):
        #return T.table(cellpadding='0', cellspacing='0')[T.span(data=T.directive('options'), render=T.directive('options'))]

        return T.select(_id=self.myselect_id,
                        name=self.myselect_id,
                        size="12",
                        multiple='true')[T.span(data=T.directive('options'),
                                             render=T.directive('options'))]#self.options()]


class LinkedSelect(AvailableSelect):
    """
        Select pour les éléments liés
    """
    docFactory = loaders.xmlfile(join(HTML_DIR, 'linked_select.xml'))
    myurl = ['link', 'linked', '?linkform=%s']
    valid_url = ['link', "valid/?linkform=%s&current=%s"]
    myselect_id = 'linked_select'

    def data_options(self, ctx, data):
        return self.form_datas['linked_select']['options']

    def _valid_url(self, ctx):
        if self.current:
            return join(_get_root_url(ctx),  *self.valid_url) % (
                                                self._type, self.current)
        else:
            return "javascript:window.alert('Il faut pas valider là !!!')"

    def render_url_slot(self, ctx, data):
        """
            rend la valeur de l'url correspondant au widget
            ainsi que les données pour la validation du formulaire
        """
        ctx.fillSlots('url', self.make_myurl(ctx))
        ctx.fillSlots('valid_url', self._valid_url(ctx))
        ctx.fillSlots('formname', self.form_datas['name'])
        ctx.fillSlots('valid_libelle', 'Validez')
        ctx.fillSlots('valid_id', "submit_%s" % self.form_datas['name'])
        ctx.fillSlots('cancel_libelle', 'Annuler')
        ctx.fillSlots('cancel_url', _get_root_url(ctx))
        return ctx.tag


class LinkForm(Page):
    """
        renvoie un formulaire d'association
    """
    docFactory = loaders.xmlfile(join(HTML_DIR, 'linkform.xml'))
    myurl = ['link', '?linkform=%s']
    children = {"available":AvailableSelect,
                "linked":LinkedSelect,
                "valid":ValidLinkForm}

    def locateChild(self, ctx, segments):
        request = inevow.IRequest(ctx)
        child, remainingSegments = Page.locateChild(self, ctx, segments)
        if request.args.has_key('linkform'):
            self._type = request.args['linkform'][0]
        else:
            raise Exception, "Erreur : il manque le type de linkform"
        self.current = None
        if request.args.has_key('current'):
            self.current = request.args['current'][0]
            self.form_datas = datas.link_form(self._type, self.current)
        else:
            self.form_datas = datas.link_form(self._type)
        if child:
            return child(self._type, self.form_datas, self.current), remainingSegments
        return self, ()

    def make_myurl(self, ctx):
        myurl = join(_get_root_url(ctx), *self.myurl) % self._type
        if self.current:
            myurl += "&current=%s" % self.current
        return myurl

    def render_choice_select(self, ctx, data):
        """ rend les données pour la mise en forme de la select avec le choix de l'élément à éditer
        """
        choice_select = ChoiceSelect(self._type, self.form_datas, self.current)
        choice_select.page = self
        return choice_select

    def render_linked_libelle(self, ctx, data):
        """ rend le titre de la select avec les éléments liés """
        return T.b[self.form_datas['libelle1']]

    def render_available_libelle(self, ctx, data):
        """ rend le titre de la select avec les éléments disponibles """
        return T.b[self.form_datas['libelle2']]

    def render_linked_select(self, ctx, data):
        """ rend les données pour la mise en forme de la select avec les éléments liés """
        linked_select = LinkedSelect(self._type, self.form_datas, self.current)
        linked_select.page = self
        return linked_select

    def render_available_select(self, ctx, data):
        """ rend la select avec les éléments disponibles """
        available_select = AvailableSelect(self._type, self.form_datas, self.current)
        available_select.page = self
        return available_select

    def render_attr_slots(self, ctx, data):
        """
            renvoie les attributs généraux du formulaire (titre...)
        """
        ctx.fillSlots('formtitle', self.form_datas['title'])
        ctx.fillSlots('url', self.make_myurl(ctx))
        return ctx.tag



