# -*- coding: utf-8 -*-
from os.path import join
from twisted.python import log
from nevow.rend import Page
from nevow import loaders, tags as T, inevow


from interface.interface.frags.message import Message
from interface.interface.data_handler import datas, valid
from interface.interface.tools import _get_root_url
from interface.interface.config import HTML_DIR

class ValidDelForm(Message):
    close_btn_code = "Refresh('listform_widget');";
    isLeaf=True

    def __init__(self, _type, form_datas):
        self._type = _type
        self.form_datas = form_datas
        super(ValidDelForm, self).__init__()

    def locateChild(self, ctx, segments):
        return self, ()

    def data_message(self, ctx, data):
        """ valide la suppression d'élément
        """
        request = inevow.IRequest(ctx)
        if request.args.has_key('todel'):
            todel = request.args['todel']
            return valid.del_(self._type, todel, self.form_datas['values'])
        return {'message':"Il manque des paramètres pour la suppression", 'alert':'_error'}

class ValidModForm(Message):
    close_btn_code = "Refresh('listform_widget');";
    isLeaf=True

    def __init__(self, _type):
         self._type = _type
         super(ValidModForm, self).__init__()
         self.form_datas = datas.list_form(self._type)

    def locateChild(self, ctx, segments):
        return self, ()

    def data_message(self, ctx, data):
        """ valide la modification d'un élément
        """
        request = inevow.IRequest(ctx)
        if request.args.has_key('mod'):
            self.item_id = request.args['mod'][0]
            self.dbkey = request.args['dbkey'][0]
            if request.args.has_key(self.dbkey):
                new_value = request.args[self.dbkey][0]
                return valid.modify(self._type, self.item_id, {self.dbkey:new_value}, self.form_datas['values'])
        return {'message':"Il manque des paramètres pour la modification", 'alert':'_error'}


class ModForm(Page):
    """
        Formulaire de modification d'élément
    """
    docFactory = loaders.xmlfile(join(HTML_DIR, 'modform.xml'))
    myurl = ['list', 'modify', '?listform=%s&tomod=%s&dbkey=%s']
    valid_url = ['list', 'modify', "modified", '?listform=%s&mod=%s&dbkey=%s']
    cancel_url = "javascript:$('mod_form_container').innerHTML = '';"

    children = {"modified" : ValidModForm,}


    def __init__(self, _type):
        self._type = _type
        super(ModForm, self).__init__()

    def locateChild(self, ctx, segments):
        request = inevow.IRequest(ctx)
        child, remainingSegments = Page.locateChild(self, ctx, segments)
        if child:
            return child(self._type), remainingSegments
        if request.args.has_key('tomod'):
            self.item_id = request.args['tomod'][0]
            self.dbkey = request.args['dbkey'][0]
            self.form_datas = datas.mod_form(self._type, self.dbkey, self.item_id)
        return self, ()

    def _get_valid_url(self, ctx):
        """ renvoie l'url de validation
        """
        return join(_get_root_url(ctx), *self.valid_url) % (self._type, self.item_id, self.dbkey)

    def render_attr_slots(self, ctx, data):
        """
            renvoie les données pour remplir les slots:
            formname : nom du formulaire
            title : titre du formulaire
        """
        ctx.fillSlots('valid_url', self._get_valid_url(ctx))
        ctx.fillSlots('formname', self.form_datas['form_name'])
        ctx.fillSlots('valid_libelle', 'Validez')
        ctx.fillSlots('valid_id', "submit_%s" % self.form_datas['form_name'])
        ctx.fillSlots('cancel_libelle', 'Annuler')
        ctx.fillSlots('cancel_url', self.cancel_url)# % (self.form_datas['form_name'],self.form_datas['form_name']))
        ctx.fillSlots('id_value', self.item_id)
        ctx.fillSlots('id_name', "item_id")
        ctx.fillSlots('dbkey', self.dbkey)
        return ctx.tag

    def data_forminput(self, ctx, data):
        """
            renvoie les données pour remplir les infos de l'input de modificatino
        """
        return [self.form_datas['_input']]


class ListForm(Page):
    """ Formulaire de suppression d'élément
    """
    docFactory = loaders.xmlfile(join(HTML_DIR, 'listform.xml'))
    myurl = ['list', '?listform=%s']
    delurl = ['list', 'delete', '?listform=%s&todel=%s']
    modurl = ['list', 'modify', '?listform=%s&tomod=%s&dbkey=%s']
    children = {#"valid":ValidDelForm,
                "delete":ValidDelForm,
                "modify":ModForm,
                }

    def locateChild(self, ctx, segments):
        request = inevow.IRequest(ctx)
        child, remainingSegments = Page.locateChild(self, ctx, segments)
        if request.args.has_key('listform'):
            self._type = request.args['listform'][0]
        else:
            raise Exception, "Erreur : il manque le type de listform"
        if request.args.has_key('ordered'):
            self.ordered = request.args['ordered'][0]
            self.form_datas = datas.list_form(self._type, self.ordered)
        else:
            self.form_datas = datas.list_form(self._type)
            self.ordered = None
        if child:
            if request.args.has_key('todel'):
                return child(self._type, self.form_datas), remainingSegments
            elif request.args.has_key('tomod') or request.args.has_key('mod'):
                return child(self._type), remainingSegments
        return self, ()

    def make_myurl(self, ctx):
        myurl = join(_get_root_url(ctx), *self.myurl) % self._type
        if self.ordered:
            myurl += "&ordered=%s" % self.ordered
        return myurl

    def data_header(self, ctx, data):
        """
            renvoie les données pour l'entête du tableau
        """
        return self.form_datas['header']

    def render_header(self, ctx, data):
        """ renvoie la ligne d'entête du tableau de données
        """
        for field in data:
            url = join(_get_root_url(ctx), *self.myurl) % self._type
            url += "&ordered=%s" % field['name']
            js_link = "javascript:callUrl('%s', [], 'content_container')" % url
            libelle = field['libelle'].capitalize()
            if field.has_key('modify'):
                libelle += " *"
            yield T.td(title=field['name'])[T.a(href=js_link)[
                                                    libelle,
                                                    T.img(src='/image/icone_tri.png'),
                                                    ]
                                            ]
        yield T.td() # pour le bouton supprimer

    def data_rows(self, ctx, data):
        """
            Renvoie les données pour les lignes du tableau
        """
        modifs = [dbkey['name'] for dbkey in self.form_datas['header'] if dbkey.has_key('modify')]
        return self.form_datas['values'], self.form_datas['keys'], modifs

    def render_rows(self, ctx, data):
        """
            renvoie les lignes du tableau
        """
        values, keys, modifs = data
        for value in values:
            yield T.tr[self._get_cells(ctx, value, keys, modifs)]

    def _get_cells(self, ctx, value, keys, modifs):
        """
            renvoie la ligne correspondant à la valeur value
        """
        for key in keys:
            if key in modifs:
                yield self._mod_btn(ctx, value[key], value['id'], key)
            else:
                yield T.td[value[key]]
        yield self._suppr_btn(ctx, value['id'])

    def _suppr_btn(self, ctx, _id):
        """
            renvoie le bouton de suppression d'un élément
        """
        url = join( _get_root_url(ctx), *self.delurl ) % (self._type, _id)
        js_link = "javascript:callUrl('%s', [], 'listform_msg_container')" % url
        return T.td[
                        T.input(_type='hidden', value=_id),
                        T.a(href=js_link)[T.div[
                            "Suppr",
                            T.img(src='/image/supprimer.gif')
                        ]]
                   ]

    def _mod_btn(self, ctx, libelle, _id, key):
        """
            renvoie le bouton de modification d'un élément
        """
        url = join( _get_root_url(ctx), *self.modurl) % (self._type, _id, key)
        container_id = "modify_entry_%s_%s" % (_id, key)
        js_link = "javascript:callUrl('%s', [], '%s')" % (url, container_id)
        return T.td[
                        T.a(href=js_link)[T.div[
                            libelle,
                        ]],
                        T.div(_id=container_id)
                   ]

    def render_attr_slots(self, ctx, data):
        """
            renvoie les données pour remplir les slots:
            formname : nom du formulaire
            title : titre du formulaire
        """
        ctx.fillSlots('url', self.make_myurl(ctx))
        return ctx.tag

