# -*- coding: utf-8 -*-
"""
    Interface d'administration du gestionnaire de base de données
    (pour le profil utilisateur dans posh)
"""
from os.path import join, dirname

from twisted.python import log

from nevow import loaders, tags as T, static, inevow, rend
from nevow.url import URL
import traceback

from interface.interface.data_handler import datas, valid
from interface.interface import tools
from interface.interface.tools import _get_root_url
from interface.interface.auth.libauth import AuthenticatedPage, Redirection
from interface.interface.config import HTML_DIR, IMG_DIR, SCRIPT_DIR, STYLE_DIR
from interface.interface.frags.menu import Menu
from interface.interface.frags.addform import AddForm
from interface.interface.frags.linkform import LinkForm
from interface.interface.frags.listform import ListForm
from interface.interface.frags.specific import IconeUpload

class RootPage(AuthenticatedPage):
    """ page de routage de l'ead """

    docFactory = loaders.xmlfile(join(HTML_DIR, 'page.xml'))
    children = {'add':AddForm(),
                'link':LinkForm(),
                'list':ListForm(),
                'image':static.File(IMG_DIR),
                'scripts':static.File(SCRIPT_DIR),
                'style':static.File(STYLE_DIR),
                'icone':IconeUpload(),
                }

    def __init__(self, auth_form, auth_server):
        super(RootPage, self).__init__(auth_form, auth_server)

    def _logout(self, ctx):
        session = inevow.ISession(ctx)
        self.reset_session(session)
        LOGOUT_URL = join(self.auth_form, 'logout', "?service=%s" % tools._get_root_url(ctx))
        return Redirection('html', LOGOUT_URL)

    def locateChild(self, ctx, segments):
        """
            scrute la requete pour renvoyer
            les pages/fragments demandés
        """
        session = inevow.ISession(ctx)
        request = inevow.IRequest(ctx)

        if 'logout' in segments:
            log.msg("++++ Logout")
            return self._logout(ctx), ()

        # authentification
        root_url = _get_root_url(ctx)
        togo = self.authenticated(request, session, root_url)
        if togo:
            return self.auth_redirect(togo)

        child, remainingSegments = rend.Page.locateChild(self, ctx, segments)
        if child:
            return child, remainingSegments
        return self, ()

    def data_js_scripts(self, ctx, data):
        """
            rend les imports de javascript
        """
        return [{'name':join('/scripts', js_file)}for js_file in tools.get_js_file_list(SCRIPT_DIR)]

    def data_css(self, ctx, data):
        """
            renvoie les paths vers les css à importer
        """
        return [{'name':join('/style', css_file)}for css_file in tools.get_css_file_list(STYLE_DIR)]

    def render_title(self, ctx, data):
        """ renvoie le titre de la page
        """
        return T.title["Profil Posh"]

    def render_icon(self, ctx, data):
        return T.link(_type="image/x-icon", href="/image/eole_icone.ico", rel="shortcut icon")

    def render_menu(self, ctx, data):
        """
            rend le menu de la page
        """
        menu = Menu()
        return ctx.tag.clear()[menu]

    def render_help(self, ctx, data):
        """ renvoie le lien vers l'aide du gestionnaire """
        HELP_URL = join(tools._get_root_url_without_port(ctx), 'doc-posh')
        js_link = "javascript:fenetre('%s', 'Aide')" % HELP_URL
        return T.a(href=js_link)[
                        T.img(src="/image/help.png")
                                ]

    def render_logout(self, ctx, data):
        """ renvoie le lien vers la déconnexion """
        session = inevow.ISession(ctx)
        LOCAL_LOGOUT_URL = join(tools._get_root_url(ctx), 'logout')
        return T.a(href=LOCAL_LOGOUT_URL)["DÉCONNEXION", T.img(src="/image/deconnecter.gif")]
