#-*-coding:utf-8-*-
"""
    Serveur xmlrpc renvoyant des données de profil
    - Ecriture des onglets obligatoires dans la base
    - Récupération des descriptions des boutons du bureau utilisateur
"""
from posh.config.config import XMLRPC_PORT as PORT

from twisted.python import log
from twisted.web import server
from twisted.internet import defer
from twisted.web.xmlrpc import XMLRPC

import urllib
from posh.perm.permission import USERCONTEXT
from posh.thumbs.initthumbs import InitThumbs
from traceback import print_exc
from random import random

allowed_users = ['admin']

def create_fr_key():
    """ crée une clé pour id de frontend """
    return str(random())

def create_magic_number():
    return str(random())

BUTTONDATAS_TEMPLATE = """<buttondatas>
%s
</buttondatas>"""

def debug(msg):
    a = file("/tmp/server-debug.log", "a")
    a.write(msg)
    a.write('\n')
    a.close()

def reload_context(context):
    print "++++ on reload le contexte dans reload_context"
    context.reload()
    return "<success>Yes</success>"

def err_reload(err):
    print "---- ERROR : erreur reload context dans err_reload"
    return "<error>No</error>"


class AuthXmlrpc(XMLRPC):
    """ pour avoir du xmlrpc authentifié """
    def __init__(self):
        XMLRPC.__init__(self)
        self.frontend_key_dict = {}
        self.magic_number_dict = {}
        self.expired_magic_number = []

    def xmlrpc_register_frontend(self, client_ip):
        """ """
        if self.frontend_key_dict.has_key(client_ip):
            log.msg("====> Erreur ----> Ce frontend est déjà enregistré dans le backend %s" % client_ip)
            key = self.frontend_key_dict[client_ip]
            return key
        else:
            key = create_fr_key()
            self.frontend_key_dict[client_ip] = str(key)
            log.msg("write_key %s " % key)
            return key

    def is_registered(self, client_ip):
        """ vérifie si le client est déjà enregistré """
        return self.frontend_key_dict.has_key(client_ip)

    def check_key(self, client_ip, client_key):
        """ teste une clé de frontend """
        if self.is_registered(client_ip):
            return self.frontend_key_dict[client_ip] == client_key

    def xmlrpc_check_key(self, client_ip, client_key):
        return str(self.check_key(client_ip, client_key))


class UserContextServer(AuthXmlrpc):
    """ Serveur Xmlrpc du gestionnaire de profil
        renvoie les decriptions des items de bureau
        et des onglets pour un utilisateur
    """

    def __init__(self):
        AuthXmlrpc.__init__(self)
        self.usercontext = USERCONTEXT

    def xmlrpc_db_reload(self):
        log.msg("#### Appel du reload de la base ")
        _deferred = defer.Deferred()
        _deferred.callback(self.usercontext)
        _deferred.addCallback(reload_context)
        _deferred.addErrback(err_reload)
        return _deferred

    def xmlrpc_get_desktop_items(self, username, user_datas):
        """ renvoie le xml décrivant les onglets de l'utilisateur
        """
        log.msg("#### Appel des items de bureau pour : %s" % username)
        log.msg("++++   Les infos additionnelles : %s " %user_datas)
        user = self.usercontext.connect_user(username, user_datas)
        log.msg("---->  Le retour xml : %s " % ''.join([desktop_item.toxml()
                for desktop_item in self.usercontext.get_desktop_items(user)]))
        return BUTTONDATAS_TEMPLATE % ''.join([desktop_item.toxml()
                for desktop_item in self.usercontext.get_desktop_items(user)])

    def xmlrpc_get_thumbs(self, username, user_datas):
        """
            récupère les onglets de l'utilisateur
        """
        log.msg("#### Appel des onglets pour : %s " % username)
        user = self.usercontext.connect_user(username, user_datas)
        # on rajoute le nom dans la description de l'onglet
        thumbs = []
        for thumb in self.usercontext.get_thumbs(user):
            thu = thumb.to_dbfield()
        #    thu.insert(0, thumb.name)
            log.msg("---->    On renvoie l'onglet : %s" %thumb.name)
            thumbs.append(thu)
        return thumbs

    def xmlrpc_export_thumb(self, username, client_ip, client_key, thumb_name, thumb_description):
        """
            rajoute un onglzet dans la base
        """
        log.msg("#FIXME: Attention ici on doit être identifié")
        log.msg(" %s %s %s" % ( username, client_ip, client_key))
        if username in allowed_users:
            if self.check_key(client_ip, client_key):
                from posh.backend.db_mapper import add_thumb, get_keys
                keys = get_keys('thumb')
                thumb_description = thumb_description[:6]
                thumb = dict([(key, thumb_description[i]) for i,key in enumerate(keys)])

                # hack pour ne pas ecrire le admin_profile_id si la page a une url externe
                # cette valeur est discriminante et permet d'identifier les pages avec widgets
                if thumb['poshtype'] == '2':
                    thumb['admin_profile_id'] = '-1'
                log.msg("++++ Insertion d'onglet : ")
                log.msg("Les clés : %s " % str(keys))
                log.msg("La description de l'onglet : %s " % thumb_description)
                try:
                    add_thumb(thumb)
                except Exception, msg:
                    print_exc()
                    return "<err>Erreur à l'écriture de l'onglet %s dans la base : %s</err>" % (thumb_name, msg.message)
                return "<msg>L'onglet %s a bien été ajouté dans la base de profil, vous pouvez désormais gérer sa diffusion.</msg>" % thumb_name
        return "<err>Erreur</err>"


if __name__ == '__main__':
    from twisted.internet import reactor
    r = UserContextServer()
    reactor.listenTCP(PORT, server.Site(r))
    reactor.run()
